# vim: set shiftwidth=4 softtabstop=4:
"""Py3o methods
"""
import atexit
import logging

from .docker_client import DockerClient

_logger = logging.getLogger(__name__)


# TODO move to docker_client directly
def find_container(name):
    """Return container object from its name
    """
    for container in DockerClient.client.containers():
        if name == container.names:
            return container


def remove_and_stop(docker_client, name):
    # TODO handle the case when the container has been created, not running
    tostop = find_container(name)
    docker_client.stop(tostop.get("Id"))
    docker_client.remove_container(tostop.get("Id"))


def start_py3o(host_fusion_port=8765, stop_at_exit=True):
    """start a py3o container"""
    docker_client = DockerClient.client
    fusion_repository = "xcgd/py3o"
    fusion_version = "1.0.0"
    fusion_name = "py3o_fusion"
    fusion_image = "{}:{}".format(fusion_repository, fusion_version)
    try:
        docker_client.pull(repository=fusion_repository, tag=fusion_version)
    except Exception as e:
        _logger.warning("Exception when trying to pull: %s", e)
    if any(
        "/{}".format(fusion_name) in container["Names"]
        for container in docker_client.containers()
    ):
        _logger.debug("%s Container already running", fusion_name)
        remove_and_stop(docker_client, fusion_name)
    # TODO handle --host-network option
    port_bindings = {8765: host_fusion_port}
    host_config = docker_client.create_host_config(
        binds=[], port_bindings=port_bindings
    )
    _logger.debug("Starting %s container", fusion_name)
    fusion = docker_client.create_container(
        image=fusion_image, host_config=host_config, name=fusion_name
    )
    _logger.debug("Starting %s container", fusion_name)
    docker_client.start(container=fusion.get("Id"))

    def stop_py3o():
        # TODO test if still exists
        _logger.info("Stopping fusion")
        docker_client.stop(fusion.get("Id"))
        _logger.info("Removing containers")
        docker_client.remove_container(fusion.get("Id"))

    if stop_at_exit:
        atexit.register(stop_py3o)
    return fusion_name, stop_py3o
