"""Function to ease connection to a postgres database
"""
import argparse
import logging
from typing import Optional

import psycopg2

from .parsing import apply, basic_parser

_logger = logging.getLogger(__name__)


def postgres_connect_parser(
    description: Optional[str], version: str
) -> argparse.ArgumentParser:
    """Create a parser for a postgres connection
    """
    parser = basic_parser(description, version)
    parser.add_argument("--database", help="the database name")
    parser.add_argument("--user", help="user name used to authenticate")
    parser.add_argument("--password", help="password used to authenticate")
    parser.add_argument(
        "--host",
        help="database host address (defaults to UNIX socket if not provided)",
    )
    parser.add_argument(
        "--port",
        help="connection port number (defaults to 5432 if not provided)",
        type=int,
    )
    return parser


def postgres_apply(namespace: argparse.Namespace):
    """Use postgres_connect_parser result to create a postgres connection
    :return: a psycopg2 connection
    """
    apply(namespace)
    connect_args = dict()
    for argument in ("database", "user", "password", "host", "port"):
        if getattr(namespace, argument):
            connect_args[argument] = getattr(namespace, argument)
    return psycopg2.connect(**connect_args)
